@@ -20,11 +20,11 @@
  */
 
 #ifndef lint
-static const char rcsid[] =
-    "@(#) $Header: /tcpdump/master/tcpslice/gmt2local.c,v 1.3 2001/10/30 17:59:46 fenner Exp $ (LBL)";
+static const char rcsid[] _U_ =
+    "@(#) $Header: /tcpdump/master/tcpslice/gmt2local.c,v 1.4 2006/08/19 08:42:43 guy Exp $ (LBL)";
 #endif
 
-#ifdef	HAVE_CONFIG_H
+#ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
@@ -36,7 +36,6 @@ static const char rcsid[] =
 #include <time.h>
 #endif
 
-#include "gnuc.h"
 #ifdef HAVE_OS_PROTO_H
 #include "os-proto.h"
 #endif
