@@ -23,7 +23,7 @@ static const char copyright[] =
     "@(#) Copyright (c) 1991, 1992, 1993, 1995, 1996, 1997, 1999, 2000\n\
 The Regents of the University of California.  All rights reserved.\n";
 static const char rcsid[] =
-    "@(#)$Header: /tcpdump/master/tcpslice/tcpslice.c,v 1.27 2001/10/30 17:59:47 fenner Exp $ (LBL)";
+    "@(#)$Header: /tcpdump/master/tcpslice/tcpslice.c,v 1.31 2006/11/30 17:26:47 raveau Exp $ (LBL)";
 #endif
 
 /*
@@ -39,7 +39,11 @@ static const char rcsid[] =
 #include <sys/file.h>
 #include <sys/stat.h>
 
-#include <net/bpf.h>
+#ifdef HAVE_NET_BPF_H
+# include <net/bpf.h>
+#else
+# include <pcap-bpf.h>
+#endif
 
 #include <ctype.h>
 #ifdef HAVE_FCNTL_H
@@ -55,7 +59,6 @@ static const char rcsid[] =
 #endif
 #include <unistd.h>
 
-#include "gnuc.h"
 #ifdef HAVE_OS_PROTO_H
 #include "os-proto.h"
 #endif
@@ -63,11 +66,16 @@ static const char rcsid[] =
 #include "tcpslice.h"
 #include "gmt2local.h"
 #include "machdep.h"
+#include "sessions.h"
 
 #ifndef HAVE_STRLCPY
 extern size_t strlcpy(char *, const char *, size_t);
 #endif
 
+#ifndef __dead
+# define __dead
+#endif
+
 /* compute a + b, store in c */
 #define	TV_ADD(a,b,c)	{ \
 	(c)->tv_sec = (a)->tv_sec + (b)->tv_sec; \
@@ -119,22 +127,21 @@ enum stamp_styles timestamp_style = TIMESTAMP_RAW;
 int is_timestamp( char *str );
 struct timeval parse_time(char *time_string, struct timeval base_time);
 void fill_tm(char *time_string, int is_delta, struct tm *t, time_t *usecs_addr);
-void get_file_range( char filename[], pcap_t **p,
-			struct timeval *first_time, struct timeval *last_time );
-struct timeval first_packet_time(char filename[], pcap_t **p_addr);
 struct timeval lowest_start_time(struct state *states, int numfiles);
+struct timeval latest_end_time(struct state *states, int numfiles);
 void get_next_packet(struct state *s);
 struct state *open_files(char *filenames[], int numfiles);
-void extract_slice(struct state *states, int numfiles, char *write_file_name,
+static void extract_slice(struct state *states, int numfiles,
+			const char *write_file_name,
 			struct timeval *start_time, struct timeval *stop_time,
 			int keep_dups, int relative_time_merge,
 			struct timeval *base_time);
 char *timestamp_to_string(struct timeval *timestamp);
-void dump_times(pcap_t **p, char filename[]);
+void dump_times(struct state *states, int numfiles);
 __dead void usage(void)__attribute__((volatile));
 
 
-pcap_dumper_t *dumper = 0;
+pcap_dumper_t *global_dumper = 0;
 
 extern  char *optarg;
 extern  int optind, opterr;
@@ -153,9 +160,8 @@ main(int argc, char **argv)
 	register char *cp;
 	char *start_time_string = 0;
 	char *stop_time_string = 0;
-	char *write_file_name = "-";	/* default is stdout */
+	const char *write_file_name = "-";	/* default is stdout */
 	struct timeval first_time, start_time, stop_time;
-	pcap_t *pcap;
 	char ebuf[PCAP_ERRBUF_SIZE];
 	struct state *states;
 
@@ -168,7 +174,7 @@ main(int argc, char **argv)
 			error("%s", ebuf);
 
 	opterr = 0;
-	while ((op = getopt(argc, argv, "dDlRrtw:")) != EOF)
+	while ((op = getopt(argc, argv, "dDe:f:lRrs:tvw:")) != EOF)
 		switch (op) {
 
 		case 'd':
@@ -179,6 +185,14 @@ main(int argc, char **argv)
 			keep_dups = 1;
 			break;
 
+		case 'e':
+			sessions_expiration_delay = atoi(optarg);
+			break;
+
+		case 'f':
+			sessions_file_format = optarg;
+			break;
+
 		case 'l':
 			relative_time_merge = 1;
 			break;
@@ -193,11 +207,20 @@ main(int argc, char **argv)
 			timestamp_style = TIMESTAMP_READABLE;
 			break;
 
+		case 's':
+			timestamp_style = TIMESTAMP_PARSEABLE;
+			sessions_init(optarg);
+			break;
+
 		case 't':
 			++report_times;
 			timestamp_style = TIMESTAMP_PARSEABLE;
 			break;
 
+		case 'v':
+			++verbose;
+			break;
+
 		case 'w':
 			write_file_name = optarg;
 			break;
@@ -239,18 +262,11 @@ main(int argc, char **argv)
 
 	if (stop_time_string)
 		stop_time = parse_time(stop_time_string, start_time);
-
 	else
-		{
-		stop_time = start_time;
-		stop_time.tv_sec += 86400*3660;	/* + 10 years; "forever" */
-		stop_time.tv_usec = 0;
-		}
-
+		stop_time = latest_end_time(states, numfiles);
 
 	if (report_times) {
-		for (; optind < argc; ++optind)
-			dump_times(&pcap, argv[optind]);
+		dump_times(states, numfiles);
 	}
 
 	if (dump_flag) {
@@ -467,41 +483,6 @@ fill_tm(char *time_string, int is_delta, struct tm *t, time_t *usecs_addr)
 }
 
 
-/* Return in first_time and last_time the timestamps of the first and
- * last packets in the given file.
- */
-void
-get_file_range( char filename[], pcap_t **p,
-		struct timeval *first_time, struct timeval *last_time )
-{
-	*first_time = first_packet_time( filename, p );
-
-	if ( ! sf_find_end( *p, first_time, last_time ) )
-		error( "couldn't find final packet in file %s", filename );
-}
-
-/* Returns the timestamp of the first packet in the given tcpdump save
- * file, which as a side-effect is initialized for further save-file
- * reading.
- */
-
-struct timeval
-first_packet_time(char filename[], pcap_t **p_addr)
-{
-	struct pcap_pkthdr hdr;
-	pcap_t *p;
-	char errbuf[PCAP_ERRBUF_SIZE];
-
-	p = *p_addr = pcap_open_offline(filename, errbuf);
-	if (! p)
-		error( "bad tcpdump file %s: %s", filename, errbuf );
-
-	if (pcap_next(p, &hdr) == 0)
-		error( "bad status reading first packet in %s", filename );
-
-	return hdr.ts;
-}
-
 
 /* Of all the files, what is the lowest start time. */
 struct timeval
@@ -518,6 +499,21 @@ lowest_start_time(struct state *states, int numfiles)
 	return min_time;
 }
 
+/* Of all the files, what is the latest end time. */
+struct timeval
+latest_end_time(struct state *states, int numfiles)
+{
+	struct timeval max_time = states->file_start_time;
+
+	while (numfiles--) {
+		if (sf_timestamp_less_than(&max_time, &states->file_stop_time)) {
+			max_time = states->file_stop_time;
+		}
+		++states;
+	}
+	return max_time;
+}
+
 /* Get the next record in a file.  Deal with end of file.
  *
  * This routine also prevents time from going "backwards"
@@ -531,6 +527,8 @@ get_next_packet(struct state *s)
 		s->pkt = pcap_next(s->p, &s->hdr);
 		if (! s->pkt) {
 			s->done = 1;
+			if (track_sessions)
+				sessions_exit();
 			pcap_close(s->p);
 		}
 	} while ((! s->done) &&
@@ -562,6 +560,8 @@ open_files(char *filenames[], int numfiles)
 		s->p = pcap_open_offline(s->filename, errbuf);
 		if (! s->p)
 			error( "bad tcpdump file %s: %s", s->filename, errbuf );
+		if (track_sessions)
+			sessions_nids_init(s->p);
 
 		this_snap = pcap_snapshot( s->p );
 		if (this_snap > snaplen) {
@@ -598,13 +598,12 @@ open_files(char *filenames[], int numfiles)
  */
 
 void
-extract_slice(struct state *states, int numfiles, char *write_file_name,
+extract_slice(struct state *states, int numfiles, const char *write_file_name,
 		struct timeval *start_time, struct timeval *stop_time,
 		int keep_dups, int relative_time_merge,
 		struct timeval *base_time)
 {
 	struct state *s, *min_state;
-	pcap_dumper_t *dumper;
 	struct timeval temp1, temp2, relative_start, relative_stop;
 	int i;
 
@@ -666,8 +665,8 @@ extract_slice(struct state *states, int numfiles, char *write_file_name,
 		get_next_packet(s);
 	}
 
-	dumper = pcap_dump_open(states->p, write_file_name);
-	if (! dumper) {
+	global_dumper = pcap_dump_open(states->p, write_file_name);
+	if (!global_dumper) {
 		error( "error creating output file %s: ",
 			write_file_name, pcap_geterr( states->p ) );
 	}
@@ -717,12 +716,20 @@ extract_slice(struct state *states, int numfiles, char *write_file_name,
 			temp1 = *stop_time;
 
 		if (sf_timestamp_less_than(&temp1, &min_state->hdr.ts)) {
-			/* We've gone beyond the end of the region
-			 * of interest ... We're done with this file.
-			 */
-			min_state->done = 1;
-			pcap_close(min_state->p);
-			break;
+			if (!sessions_count) {
+				/* We've gone beyond the end of the region
+				 * of interest ... We're done with this file.
+				 */
+				if (track_sessions)
+					sessions_exit();
+				min_state->done = 1;
+				pcap_close(min_state->p);
+				break;
+			} else {
+				/* We need to wait for the sessions to close */
+				bonus_time = 1;
+				*stop_time = min_state->file_stop_time;
+			}
 		}
 
 		if (relative_time_merge) {
@@ -730,19 +737,26 @@ extract_slice(struct state *states, int numfiles, char *write_file_name,
 			TV_ADD(&temp1, base_time, &min_state->hdr.ts);
 		}
 
-		/* Dump it, unless it's a duplicate. */
-		if ( keep_dups ||
-		     min_state == last_state ||
-		     memcmp(&last_hdr, &min_state->hdr, sizeof(last_hdr)) ||
-		     memcmp(last_pkt, min_state->pkt, last_hdr.caplen) ) {
-			pcap_dump((u_char *) dumper, &min_state->hdr, min_state->pkt);
+#ifdef HAVE_LIBNIDS
+		/* Keep track of sessions, if specified by the user */
+		if (track_sessions)
+			nids_pcap_handler((u_char *)min_state->p, &min_state->hdr, (u_char *)min_state->pkt);
+#endif
 
-			if ( ! keep_dups ) {
-				last_state = min_state;
-				last_hdr = min_state->hdr;
-				memcpy(last_pkt, min_state->pkt, min_state->hdr.caplen);
+		/* Dump it, unless it's a duplicate. */
+		if (!bonus_time)
+			if ( keep_dups ||
+			     min_state == last_state ||
+			     memcmp(&last_hdr, &min_state->hdr, sizeof(last_hdr)) ||
+			     memcmp(last_pkt, min_state->pkt, last_hdr.caplen) ) {
+				pcap_dump((u_char *) global_dumper, &min_state->hdr, min_state->pkt);
+
+				if ( ! keep_dups ) {
+					last_state = min_state;
+					last_hdr = min_state->hdr;
+					memcpy(last_pkt, min_state->pkt, min_state->hdr.caplen);
+				}
 			}
-		}
 
 		get_next_packet(min_state);
 	}
@@ -804,16 +818,14 @@ timestamp_to_string(struct timeval *timestamp)
  */
 
 void
-dump_times(pcap_t **p, char filename[])
+dump_times(struct state *states, int numfiles)
 {
-	struct timeval first_time, last_time;
-
-	get_file_range( filename, p, &first_time, &last_time );
-
-	printf( "%s\t%s\t%s\n",
-		filename,
-		timestamp_to_string( &first_time ),
-		timestamp_to_string( &last_time ) );
+	for (; numfiles--; states++) {
+		printf( "%s\t%s\t%s\n",
+			states->filename,
+			timestamp_to_string( &states->file_start_time ),
+			timestamp_to_string( &states->file_stop_time ) );
+	}
 }
 
 __dead void
@@ -822,8 +834,10 @@ usage(void)
 	extern char version[];
 
 	(void)fprintf(stderr, "Version %s\n", version);
-	(void)fprintf(stderr,
-"Usage: tcpslice [-DdlRrt] [-w file] [start-time [end-time]] file ... \n");
+        (void)fprintf(stderr,
+		      "Usage: tcpslice [-DdlRrtv] [-w file]\n"
+		      "                [ -s types [ -e seconds ] [ -f format ] ]\n"
+		      "                [start-time [end-time]] file ... \n");
 
 	exit(-1);
 }
